import os
import aiohttp
import asyncio
import json
import sys
from json_repair import repair_json

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from fsd.util.portkey import AIGateway
from fsd.log.logger_config import get_logger
from fsd.util.utils import log_error
logger = get_logger(__name__)

class PromptImageUrlAgent:
    def __init__(self, repo):
        """
        Initialize the PromptImageUrlAgent with the repository.

        Args:
            repo: The repository object.
        """
        self.repo = repo
        self.max_tokens = 4096
        self.ai = AIGateway()

    async def get_image_links(self, idea):
        """
        Extract image links from the given idea.

        Args:
            idea (str): The user's idea or request.

        Returns:
            dict: JSON response with the extracted image links.
        """
        logger.debug("\n #### PromptImageUrlAgent is checking for image links")
        prompt = (
            "Analyze the provided development plan and extract any image links (png, jpg, jpeg, webp) that can be converted to base64. "
            "Only include valid image links. Do not invent or hallucinate links. "
            "If the user provides an incomplete URL (e.g., missing 'https://', 'www', etc.), correct it to a fully valid URL. "
            "Provide the response as a JSON object with the following format:"
            "{\n"
            "    \"assets_link\": [\"https://example.com/image1.png\", \"https://www.example.com/image2.jpg\"]\n"
            "}\n\n"
            "If no valid image links are found, return an empty list:"
            "{\n"
            "    \"assets_link\": []\n"
            "}\n\n"
            "Return only valid JSON without any additional text or Markdown formatting."
        )

        messages = [
            {
                "role": "system",
                "content": prompt
            },
            {
                "role": "user",
                "content": f"This is the user's request to do:\n{idea}\n"
            }
        ]

        try:
            logger.debug("\n #### PromptImageUrlAgent is sending request to AI for image link extraction")
            response = await self.ai.prompt(messages, self.max_tokens, 0.2, 0.1)
            logger.debug("\n #### PromptImageUrlAgent has received response from AI")
            return json.loads(response.choices[0].message.content)
        except json.JSONDecodeError:
            logger.debug("\n #### PromptImageUrlAgent encountered JSON decode error, attempting repair")
            good_json_string = repair_json(response.choices[0].message.content)
            plan_json = json.loads(good_json_string)
            return plan_json
        except Exception as e:
            error_message = f" #### PromptImageUrlAgent encountered an error: `{e}`"
            logger.error(error_message)
            log_error(error_message, str(e))
            return {
                "reason": str(e)
            }

    async def process_image_links(self, idea):
        logger.debug("\n #### PromptImageUrlAgent is starting image link extraction process")
        return await self.get_image_links(idea)
